@@ -16,28 +16,29 @@ void ConnectionsConsumer::run()
         if (d_storage.empty())
             break;
 
-        Record &record = d_storage.fetch(); // makes available the next
-                                            // record. Since this is the
+        Record *record = d_storage.fetch(); // makes available the next
+                                            // record-> Since this is the
                                             // only thread fetching records
                                             // there's no need to copy the
                                             // record at this point
         d_storage.produceNotify();
+
                                             // process the incoming protocol
-                                            // data, calls tcp, udp or icmp
-        (this->*s_handler[record.protocol()])(record);  
+                                            // data, calls overridden
+                                            // tcp.process(), udp.process() or
+                                            // icmp.process() through
+                                            // ipbase.process()
+        d_handler[record->protocol()]->process( RecordPtr{ record } );  
     }
 
     if (d_options.realTime())
         cleanupThread.join();
         
-    d_logType = EOP;
-
     time_t eop = time(0) + 1;
 
     cleanupICMP_UDP(eop);                   // what's still in the maps is
                                             // there at EOP
-
-    cleanup(eop, d_tcpMutex, d_tcp, &ConnectionsConsumer::logTCP_UDP, "tcp");
+    d_tcp.cleanup(eop);
 }
 
 
